

/**
 * stack is an (imginary) "array"; where you're 
 * inserting/removing things from the end; push/pop.
 * LIFO (last in-first out)
 * [1,2,3,4,5]
 * after push(7); [1,2,3,4,5,7];
 * doing pop on [1,2,3,4,5] will return 5.
 */
class MyStack {
    int[] arr;
    int top;
    public MyStack(int siz){
        arr = new int[siz];
        top = -1;
    }    
    public boolean isEmpty(){
        return top < 0;
    }
    public void push(int n){
        arr[++top] = n;
    }
    public int pop(){
        int n = arr[top];
        top--;
        return n;
    }
}

/**
 * FIFO (first in - first out)
 * insert 7 on [1,2,3,4] will do [1,2,3,4,7],
 * remove on [1,2,3,4] will remove 1
 *  and leave [2,3,4];
 */
class MyQueue {
    int[] arr;
    int head,tail;
    public MyQueue(int siz){
        arr = new int[siz+1];
        head = tail = 0;
    }
    public boolean isEmpty(){
        return head == tail;
    }
    public void insert(int n){
        arr[head] = n;
        head = (head + 1) % arr.length;
    }
    public int remove(){
        int n = arr[tail];
        tail = (tail + 1) % arr.length;
        return n;
    }
}


public class MyList {
    
    int[] arr;
    int head,tail;
    public MyList(int siz){
        arr = new int[siz+1];
        head = tail = 0;
    }
    public boolean isEmpty(){
        return head == tail;
    }
    // [1,2,3] push 7, will result in [1,2,3,7]
    public void push(int n){
        arr[head] = n;
        head = (head + 1) % arr.length;
    }

    // [1,2,3] pop will result in 3, leaving [1,2];
    public int pop(){
        head = (arr.length + head - 1) % arr.length;
        return arr[head];
    }

    // [1,2,3] shift will result in 1, leaving [2,3];
    public int shift(){
        int n = arr[tail];
        tail = (tail + 1) % arr.length;
        return n;
    }
    // [1,2,3] unshift 7, will result in [7,1,2,3]
    public void unshift(int n){
        tail = (arr.length + tail - 1) % arr.length;
        arr[tail] = n;
    }

    // get the ith element in list.
    // [1,2,3], 2nd element is 2.
    public int get(int i){
        return arr[(tail + i) % arr.length];
    }

    // sets ith element.
    public void set(int i,int n){
        arr[(tail + i) % arr.length] = n;
    }

    public int size(){
        return ((arr.length + head) - tail) % arr.length;
    }

    public String toString(){
        String out = "[";
        for(int i=0;i<size()-1;i++){
            out += ""+get(i)+",";
        }
        out += ""+get(size()-1)+"]";
        return out;
    }

    public static void main(String[] args){
        System.out.println("Hello World!");
    
        /*
        MyStack s = new MyStack(100);
        for(int i=0;i<10;i++){
            s.push(i*i);
            System.out.println("pushed: "+(i*i));
        }

        while(!s.isEmpty()){
            int n = s.pop();
            System.out.println("poped: "+n);
        }
        */

        /*
        MyQueue s = new MyQueue(6);
        for(int i=0;i<5;i++){
            s.insert(i*i);
            System.out.println("insert: "+(i*i));
        }
        while(!s.isEmpty()){
            int n = s.remove();
            System.out.println("remove: "+n);
        }
        for(int i=0;i<5;i++){
            s.insert(i*i);
            System.out.println("insert: "+(i*i));
        }
        while(!s.isEmpty()){
            int n = s.remove();
            System.out.println("remove: "+n);
        }
        */

        MyList l = new MyList(20);
        for(int i=0;i<5;i++){
            l.push(i*i);
            System.out.println("pushed:"+(i*i)+"; list:"+l);
        }

        for(int i=0;i<5;i++){
            l.unshift(i*i);
            System.out.println("unshift:"+(i*i)+"; list:"+l);
        }

        for(int i=0;i<5;i++){
            l.push(i*i);
            System.out.println("pushed:"+(i*i)+"; list:"+l);
        }

        for(int i=0;i<4;i++){
            int n = l.shift();
            System.out.println("shift:"+n+"; list:"+l);
        }
        for(int i=0;i<6;i++){
            int n = l.pop();
            System.out.println("pop:"+n+"; list:"+l);
        }
        for(int i=0;i<l.size();i++){
            System.out.println("["+i+"]: "+l.get(i));
        }
        while(!l.isEmpty()){
            int n = l.shift();
            System.out.println("shift:"+n+"; list:"+l);
        }
    }
}

