
import java.util.*;
import java.io.*;

/**
 * Product value object
 *  ie: data structure without any functionality.
 */
class Product {
    
    public int id;
    public String description;
    public double price;
    public double size;
    public String color;
    public String vendor;

    public Product(){
        id = 0;
        description = "";
        price = 0;
        size = 0;
        color = "";
        vendor = "";    
    }
    public Product(  
        int id,
        String description,
        double price,
        double size,
        String color,
        String vendor){
        
        this.id = id;
        this.description = description;
        this.price = price;
        this.size = size;
        this.color = color;
        this.vendor = vendor;
    }

    public String getDescription(){
        return description;
    }
    public void setDescription(String description){
        this.description = description;
    }

    public String toString(){
        return "id:"+id+"\n"+
                "description:"+description+"\n"+
                "price:"+price+"\n"+
                "size:"+size+"\n"+
                "color:"+color+"\n"+
                "vendor:"+vendor+"\n";
    }
}

/**
 * Data access object for Product (DAO).
 * Helper class to read/write Product to data store.
 */
class ProductDAO {

    public Vector data;
    public String file;
    
    public ProductDAO(String f){
        file = f;
        data = new Vector();
    }

    // add new product
    public void add(Product o){
        data.add(o);
    }

    // remove product wtih id
    public int remove(int id){
        return 0;
    }

    public void update(Product o){

    }

    public Product get(int id){
        return null;
    }

    public Vector find(String s){
        return new Vector();
    }

    public Vector list(){
        return data;
    }

    public void load() throws IOException {
        data = new Vector();
        BufferedReader r = new BufferedReader(new FileReader(file));
        String line = r.readLine();
        while(line != null){
            line = line.trim();
            if(line.length() <= 0)
                continue;
            String[] arr = line.split(",");
            Product p = new Product();
            p.id = Integer.parseInt(arr[0]);
            p.description = arr[1];
            p.price = Double.parseDouble(arr[2]);
            p.size = Double.parseDouble(arr[3]);
            p.color = arr[4];
            p.vendor = arr[5];
            data.add(p);
            line = r.readLine();
        }
        r.close();
    }

    public void save() throws IOException {
        PrintWriter out = new PrintWriter(new FileWriter(file));
        for(int i=0;i<data.size();i++){
            Product p = (Product)data.elementAt(i);
            out.println(
                ""+p.id+","+
                ""+p.description+","+
                ""+p.price+","+
                ""+p.size+","+
                ""+p.color+","+
                ""+p.vendor
            );
        }
        out.close();    
    }
}

class ProductUI {
    
    public static String prompt(BufferedReader br,String q,String d) throws IOException {
        System.out.print(""+q+"["+d+"]: ");
        String a = br.readLine();
        if(a == null)
            return d;
        if(a.trim().length() == 0)
            return d;
        return a;
    }

    public static Product newProduct(BufferedReader br) throws IOException {
        Product p = new Product();
        System.out.println("Add New Product");
        p.id = Integer.parseInt(prompt(br,"Id","0"));
        p.description = prompt(br,"Description","");
        p.price = Double.parseDouble(prompt(br,"Price","0"));
        p.size = Double.parseDouble(prompt(br,"Size","0"));
        p.color = prompt(br,"Color","");
        p.vendor = prompt(br,"Vendor","");
        return p;
    }

}

/*
 * Project to manage a database of products.
 */
public class Proj {


    public static void main(String[] args) throws Throwable {
        String line;
        BufferedReader br = new BufferedReader(
            new InputStreamReader(System.in));

        ProductDAO dao = new ProductDAO("db.txt");

        System.out.print("> ");
        line = br.readLine();
        while(line != null){
            line = line.trim();    //remove whitespace.
            if(line.equals("exit"))
                 break;
            
            if(line.equals("time")){
                System.out.println("time now is: "+new Date());
            }else if(line.equals("add")){
                Product p = ProductUI.newProduct(br);
                System.out.println("You're entered:\n-----\n"+p+"-----\n");
                if(ProductUI.prompt(br,"Add? Y/N","N").equalsIgnoreCase("y")){
                    dao.add(p);
                    dao.save();
                }

            }else if(line.equals("list")){
                System.out.println("LIST ALL");
                Vector v = dao.list();
                for(int i=0;i<v.size();i++){
                    Product p = (Product)v.elementAt(i);
                    System.out.println(""+p+"------\n");
                }

            }else if(line.equals("load")){
                dao.load();

            }else if(line.length() > 0){
                System.out.println("unrecognized command ["+line+"]");
            }

            System.out.print("> ");
            line = br.readLine();
        }
    }    
}

